package com.bgy.sms.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.bgy.sms.channel.bgy.config.BgySMSConfig;
import com.bgy.sms.channel.bgy.dto.CLBizResponse;
import com.bgy.sms.channel.bgy.service.BgySmsService;
import com.bgy.sms.channel.dmHub.config.DmHubConfig;
import com.bgy.sms.channel.dto.*;
import com.bgy.sms.config.ResponseCode;
import com.bgy.sms.repository.domain.*;
import com.bgy.sms.repository.mapper.MessageMapper;
import com.bgy.sms.service.*;
import com.bgy.sms.service.bean.TemplateChangeBean;
import com.bgy.util.id.UUIDUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@SuppressWarnings("Duplicates")
@Service
public class MessageServiceImpl implements MessageService {

    private static final Logger log = LoggerFactory.getLogger(MessageServiceImpl.class);

    @Autowired
    private SysBatchServiceImpl sysBatchService;
    @Autowired
    private SmsTemplateService smsTemplateService;
    @Autowired
    private BgySmsService bgySmsService;
    @Autowired
    private DmBatchService dmBatchService;
    @Autowired
    private DmCodeService dmCodeService;
    @Autowired
    private SysRecordService sysRecordService;
    @Autowired
    private XiaoShuService xiaoShuService;
    @Autowired
    private MessageMapper messageMapper;

    private static final String REJECTED = "rejected";    //  创建模板响应数据
    private static final String APPROVED = "approved";    //  创建模板响应数据

    /**
     * 短信模板创建
     *
     * @param requestDTO
     * @return
     */
    @Override
    public DmHubResponse createTemplate(DmHubTemplateRequest requestDTO) {
        log.info("进入创建短信模板业务逻辑，入参：{}", requestDTO);

        DmHubResponse responseDTO = new DmHubResponse(ResponseCode.SUCCESS);
        String templateId = null;
        try {
            String templateContent = requestDTO.getTemplateContent();
            String signature = requestDTO.getSignature();
            String smsType = requestDTO.getSmsType();
            String tenantId = requestDTO.getTenantId();
            templateId = requestDTO.getTemplateId();
            String templateName = requestDTO.getTemplateName();
            // 替换模板格式，保存适合碧桂园发送短信的模板，避免发送时修改模板格式
            TemplateChangeBean bean = dmHub2BgyTemplateSend(templateContent);
            String bgySendStr = bean.getUpSendStr();
            String upCreateStr = bean.getUpCreateStr();
            String params = bean.getParams().toString();
            SmsTemplateInfo dbInfo = smsTemplateService.selectOne(new EntityWrapper<SmsTemplateInfo>()
                    .eq("dm_template_id", templateId)
                    .eq("tenant_id", tenantId)
            );
            if (dbInfo != null) {
                log.error("Dm Hub请求创建模板，模板id已存在，模板id：【{}】", templateId);
                return new DmHubResponse(ResponseCode.TEMPLATE_ALREADY_EXISTED);
            }
            String areaId = messageMapper.selectAreaId();
            if (StringUtils.isBlank(areaId)) {
                return new DmHubResponse("500", "未获取到项目ID，请检查area_info配置");
            }
            SmsTemplateInfo info = new SmsTemplateInfo();
            info.setContent(templateContent);
            info.setType(smsType);
            info.setDmTemplateId(templateId);
            info.setTenantId(tenantId);
            info.setId(UUIDUtils.getId());
            info.setSignature(signature);
            info.setTemplateName(templateName);
            info.setDateCreated(new Date());
            info.setLastUpdated(new Date());
            info.setLastUpdated(new Date());
            info.setUpContent(bgySendStr);
            info.setParams(params);
            boolean insert = smsTemplateService.insert(info);
            if (!insert) {
                return new DmHubResponse(ResponseCode.SYSTEM_ERROR);
            } else {
                String pkID = info.getId();
                String TEMPLATETYPE = "";
                if ("marketing".equals(smsType)) {
                    TEMPLATETYPE = "52";
                } else {
                    TEMPLATETYPE = "49";
                }
                CLBizResponse response = new CLBizResponse();
                String account = xiaoShuService.selectUserIdByTemplate(templateId);
                if (StringUtils.isBlank(account)) {
                    return new DmHubResponse(ResponseCode.NO_USER_ID);
                }
                response = bgySmsService.createTemplate(upCreateStr, areaId, account, TEMPLATETYPE, pkID);
                String code = response.getCode();
                String msg = response.getMsg();
                if (code.equals(ResponseCode.SUCCESS.getCode())) {
                    SmsTemplateInfo updateInfo = new SmsTemplateInfo();
                    updateInfo.setTemplateRecordId(msg);
                    updateInfo.setId(pkID);
                    smsTemplateService.updateById(updateInfo);
                    return new DmHubResponse(ResponseCode.SUCCESS);
                } else {
                    return new DmHubResponse(code, msg);
                }
            }
        } catch (Exception exception) {
            log.error("创建模板业务逻辑异常，错误信息", exception);
            if (!StringUtils.isBlank(templateId)) {
            }
            responseDTO = new DmHubResponse(ResponseCode.SYSTEM_ERROR);
        }
        return responseDTO;
    }


    /**
     * 将DM HUb的模板装成可以简单模式模板，用户后续参数封装
     *
     * @param content
     * @return
     */
    private TemplateChangeBean dmHub2BgyTemplateSend(String content) {
        String regex = "((?:\\$\\{surl.*'\\}))|\\$\\{[^}]+\\}";
        Pattern pattern = Pattern.compile(regex);
        Matcher matcher = pattern.matcher(content);

        List<String> groups = new ArrayList<>();
        while (matcher.find()) {
            String group = matcher.group();
            groups.add(group);
        }
        JSONArray arr = new JSONArray();
        if (groups.isEmpty()) {
            return new TemplateChangeBean(content, arr, content);
        }

        String upCreateStr = content;
        String upSendStr = content;
        for (String g : groups) {
            String t = g;
            g = Matcher.quoteReplacement(g);
            g = escapeRegex(g);
            upSendStr = upSendStr.replaceAll(g, "\\$\\$");
            if (t.contains("${surl!")) {
                // 短链没传长度，固定长度20
                String str = DmHubConfig.shortUrl + "\\$$";
                str = java.util.regex.Matcher.quoteReplacement(str);
                upCreateStr = upCreateStr.replaceAll(g, str);

            } else {
                upCreateStr = upCreateStr.replaceAll(g, "\\$\\$");

            }
        }
        arr.addAll(groups);
        return new TemplateChangeBean(upSendStr, arr, upCreateStr);
    }

    private boolean isNumeric(String str) {
        if (StringUtils.isBlank(str)) {
            return false;
        }
        for (int i = 0; i < str.length(); i++) {
            System.out.println(str.charAt(i));
            if (!Character.isDigit(str.charAt(i))) {
                return false;
            }
        }
        return true;
    }

    private static String escapeRegex(String s) {
        String[] symbols = new String[]{"\\/", "\\[", "\\]", "\\(", "\\)", "\\{", "\\}", "\\?", "\\+", "\\*", "\\|", "\\.", "\\^", "\\-", "\\^"};
        if (s != null) {
            for (String sym : symbols) {
                s = s.replaceAll(sym, "\\" + sym);
            }
        }
        return s;
    }

    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
    public DmHubResponse send(DmHubSendRequest request) {
        try {
            String dmHubBatchId = request.getBatchId();
            String channelAccount = request.getChannelAccount();
            String templateId = request.getTemplateId();
            String audienceIdType = request.getAudienceIdType();
            JSONObject data = request.getData();
            // 1、获取模板信息
            SmsTemplateInfo templateInfo = smsTemplateService.selectOne(new EntityWrapper<SmsTemplateInfo>().eq("dm_template_id", templateId));
            if (null == templateInfo) {
                log.error("未获取到短信模板");
                return new DmHubResponse("999", "未获取到短信模板");
            }
            // 2、根据请求信息获取用户的手机号码（此处不考虑配置DMHUB系统不是SMS的情况），直接拿请求体中的_audienceId字段值，该值为手机号码
            // 模板参数占位符
            if (templateInfo.getStatus().equals("abnormal")) {
                return new DmHubResponse("1001", "短信模板正在审核中，请稍后重试");
            }

            String areaId = messageMapper.selectAreaId();
            if (StringUtils.isBlank(areaId)) {
                return new DmHubResponse("500", "未获取到项目ID，请检查area_info配置");
            }
            String params = templateInfo.getParams();
            JSONArray paramsArr = JSONArray.parseArray(params);
            String smsType = templateInfo.getType();
            // 封装参数
            List<String> paramList = new ArrayList<>();
            String mobile = data.getString("_audienceId");
            data.remove("name");
            data.remove("smsMessage");
            data.remove("inWhiteList");
            data.remove("isRealtime");
            data.remove("id");
            data.remove("_audienceId");

            log.info("data信息：{}", data);
            if (!data.isEmpty()) {
                List<String> paramsList = new ArrayList<>();
                StringBuffer sb = new StringBuffer();
                Map<String, String> retMap = new HashMap<>();
                getKeyAndValue(data, paramsList, sb, retMap);
                for (Object parm : paramsArr) {
                    String s = parm.toString();
                    for (String newKeyStr : paramsList) {
                        String newKey = newKeyStr.substring(1);
                        if (s.contains(newKey)) {
                            paramList.add(retMap.get(newKeyStr));
                        }
                    }
                }
            }
            String type = templateInfo.getType();
            // 3、记录批次信息
            SysBatchInfo sysBatchInfo = new SysBatchInfo();
            String sysBatchId = UUIDUtils.getId();
            sysBatchInfo.setBatchId(sysBatchId);
            sysBatchInfo.setDmBatchId(dmHubBatchId);
            sysBatchInfo.setId(UUIDUtils.getId());
            sysBatchInfo.setDmTemplateId(templateId);
            sysBatchInfo.setReport("1");
            sysBatchInfo.setSmsNum(1);
            sysBatchInfo.setDateCreated(new Date());
            sysBatchInfo.setLastUpdated(new Date());
            boolean insert = sysBatchService.insert(sysBatchInfo);

            CLBizResponse response = new CLBizResponse();
            String content = null;
            if (paramsArr == null || paramsArr.isEmpty()) {
                // 固定短信发送
                content = templateInfo.getUpContent();
            } else {
                // 变量短信发送
                String upContent = templateInfo.getUpContent();
                content = getMsg(upContent, paramList);
            }

            String api = "";
            if ("marketing".equals(smsType)) {
                api = "SendYXSMS";
            } else {
                api = "SendNotifySMS";
            }
            response = bgySmsService.sendSms(mobile, content, areaId, channelAccount, api);
            //4.短信发送记录
            SysRecordInfo sysRecordInfo = new SysRecordInfo();
            sysRecordInfo.setId(UUIDUtils.getId());
            String sysRecordInfoBatchId = UUIDUtils.getId();
            sysRecordInfo.setSysBatchId(sysRecordInfoBatchId);
            sysRecordInfo.setMobile(mobile);
            sysRecordInfo.setChargeNum(1);
            sysRecordInfo.setContent(content);
            Date date = new Date();
            sysRecordInfo.setDateCreated(date);
            sysRecordInfo.setLastUpdated(date);
            sysRecordInfo.setAreaId(channelAccount);


            String code = response.getCode();
            if (code.equals(ResponseCode.SUCCESS.getCode())) {
                sysRecordInfo.setStatus(ResponseCode.SUCCESS.getCode());
                sysRecordInfo.setStatusDesc(response.getMsg());
                sysRecordService.insert(sysRecordInfo);
                return new DmHubResponse(ResponseCode.SUCCESS);
            } else {
                sysRecordInfo.setStatus(ResponseCode.UPSTREAM_FAIL.getCode());
                sysRecordInfo.setStatusDesc(response.getMsg());
                boolean a = sysRecordService.insert(sysRecordInfo);
                return new DmHubResponse(code, response.getMsg());
            }

        } catch (Exception e) {
            log.error("调用短信发送逻辑错误", e);
            return new DmHubResponse("999", "插件系统异常");
        }
    }

    @Override
    public DmHubResponse sendCode(DmHubCodeRequest params) {
        String mobile = params.getMobile();
        String code = params.getCode();
        CLBizResponse response = new CLBizResponse();
        String content = "您的验证码是：" + code;
        try {
            String areaId = messageMapper.selectAreaId();
            if (StringUtils.isBlank(areaId)) {
                return new DmHubResponse("500", "未获取到项目ID，请检查area_info配置");
            }
            DmCodeInfo codeInfo = new DmCodeInfo();
            codeInfo.setId(UUIDUtils.getId());
            codeInfo.setAreaId(areaId);
            codeInfo.setCode(code);
            codeInfo.setMobile(mobile);
            codeInfo.setDateCreated(new Date());
            dmCodeService.insert(codeInfo);
            response = bgySmsService.sendSms(mobile, content, areaId, null, null);
            String retCode = response.getCode();
            if (retCode.equals(ResponseCode.SUCCESS.getCode())) {
                return new DmHubResponse(ResponseCode.SUCCESS);
            } else {
                return new DmHubResponse(retCode, response.getMsg());
            }
        } catch (Exception e) {
            log.error("发送短信异常：", e);
            return new DmHubResponse("999", "发送通道异常");
        }

    }

    /**
     * 获取可以直接用户发送的短信内容
     *
     * @param upContent
     * @param params
     * @return
     */
    private String getMsg(String upContent, List<String> params) {
        for (String info : params) {
            upContent = upContent.replaceFirst("\\$\\$", info);
        }
        return upContent;
    }


    @Override
    public DmHubResponse batchSendBatch(DmHubBatchSendRequest request) {
        String batchId = request.getBatchId(); // DM hub 批次号
        String templateId = request.getTemplateId(); // 此次短信对应的模板id
        List<JSONObject> data = request.getData();
        String channelAccount = request.getChannelAccount();
        SmsTemplateInfo templateInfo = smsTemplateService.selectOne(new EntityWrapper<SmsTemplateInfo>().eq("dm_template_id", templateId));
        if (null == templateInfo) {
            return new DmHubResponse("999", "短信插件未获取到模板信息");
        }

        if (templateInfo.getStatus().equals("abnormal")) {
            return new DmHubResponse("1001", "短信模板正在审核中，请稍后重试");
        }
        String api = "";
        if ("marketing".equals(templateInfo.getType())) {
            api = "SendYXSMS";
        } else {
            api = "SendNotifySMS";
        }
        //模板流水号
        String templateRecordId = templateInfo.getTemplateRecordId();
        String smsType = templateInfo.getType();
        // 模板参数占位符
        String params = templateInfo.getParams();
        JSONArray paramsArr = JSONArray.parseArray(params);

        // 封装参数
        List<String> mobileList = new ArrayList<>();
        Map<String, List<String>> paramsMap = new HashMap();
        int i = 0;
        for (JSONObject json : data) {
            i++;
            Set<String> keys = json.keySet();
            String mobile = json.getString("_audienceId");
            json.remove("name");
            json.remove("smsMessage");
            json.remove("inWhiteList");
            json.remove("isRealtime");
            json.remove("id");
            json.remove("_audienceId");
            mobileList.add(mobile);
            List<String> list = new ArrayList<>();
            if (!json.isEmpty()) {
                List<String> paramsList = new ArrayList<>();
                StringBuffer sb = new StringBuffer();
                Map<String, String> retMap = new HashMap<>();
                getKeyAndValue(json, paramsList, sb, retMap);
                for (Object parm : paramsArr) {
                    String s = parm.toString();
                    for (String newKeyStr : paramsList) {
                        String newKey = newKeyStr.substring(1);
                        if (s.contains(newKey)) {
                            list.add(retMap.get(newKeyStr));
                        }
                    }
                    paramsMap.put(mobile, list);
                }
            } else {
                if (i != data.size()) {
                    paramsMap.put(mobile, new ArrayList<>());
                }
            }
        }

        DmBatchInfo dmInfo = new DmBatchInfo();
        dmInfo.setDmBatchId(batchId);
        dmInfo.setDmTemplateId(templateId);
        dmInfo.setSmsNum(data.size());
        dmInfo.setId(UUIDUtils.getId());
        dmInfo.setDateCreated(new Date());
        dmInfo.setLastUpdated(new Date());
        dmBatchService.insert(dmInfo);

        SysBatchInfo info = new SysBatchInfo();
        String sysBatchId = UUIDUtils.getId();
        info.setId(sysBatchId);
        info.setDmBatchId(batchId);
        info.setSmsNum(data.size());
        info.setDmTemplateId(templateId);
        info.setBatchId(sysBatchId);
        info.setDateCreated(new Date());
        info.setLastUpdated(new Date());
        sysBatchService.insert(info);
        CLBizResponse response = new CLBizResponse();

        //4.短信发送记录
        SysRecordInfo sysRecordInfo = new SysRecordInfo();
        sysRecordInfo.setId(UUIDUtils.getId());
        String sysRecordInfoBatchId = UUIDUtils.getId();
        sysRecordInfo.setSysBatchId(sysRecordInfoBatchId);
        Date date = new Date();
        sysRecordInfo.setDateCreated(date);
        sysRecordInfo.setLastUpdated(date);
        sysRecordInfo.setAreaId(channelAccount);
        // TODO 获取AreaId;
        String areaId = messageMapper.selectAreaId();
        if (StringUtils.isBlank(areaId)) {
            return new DmHubResponse("500", "未获取到项目ID，请检查area_info配置");
        }

        if (paramsArr == null || paramsArr.isEmpty()) {
            List<JSONObject> list = new ArrayList<>();
            StringBuilder mobiles = new StringBuilder();
            String content = templateInfo.getUpContent();
            for (String mobile : mobileList) {
                JSONObject jsonObject = new JSONObject();
                mobiles.append(mobile + ",");
                jsonObject.put("Mobile", mobile);
                jsonObject.put("Content", content);
                list.add(jsonObject);
            }
            try {
                if (!list.isEmpty()) {
                    response = bgySmsService.sendBatchSms(list, areaId, channelAccount, api, templateRecordId);
                    String code = response.getCode();
                    if (code.equals(ResponseCode.SUCCESS.getCode())) {
                        sysRecordInfo.setChargeNum(list.size());
                        sysRecordInfo.setMobile(mobiles.deleteCharAt(mobiles.length() - 1).toString());
                        sysRecordInfo.setContent(content);
                        sysRecordInfo.setStatus(ResponseCode.SUCCESS.getCode());
                        sysRecordInfo.setStatusDesc(response.getMsg());
                        sysRecordService.insert(sysRecordInfo);
                        return new DmHubResponse(ResponseCode.SUCCESS.getCode(), ResponseCode.SUCCESS.getMsg());
                    } else {
                        sysRecordInfo.setChargeNum(list.size());
                        sysRecordInfo.setMobile(mobiles.deleteCharAt(mobiles.length() - 1).toString());
                        sysRecordInfo.setContent(content);

                        sysRecordInfo.setStatus(ResponseCode.UPSTREAM_FAIL.getCode());
                        sysRecordInfo.setStatusDesc(response.getMsg());
                        boolean a = sysRecordService.insert(sysRecordInfo);
                        return new DmHubResponse(code, response.getMsg());
                    }
                }
            } catch (Exception e) {
                log.error("调用碧桂园逻辑错误", e);
                return new DmHubResponse("999", "插件服务系统异常");
            }

        } else {
            // 变量短信
            Set<Map.Entry<String, List<String>>> entries = paramsMap.entrySet();
            List<JSONObject> list = new ArrayList<>();
            StringBuilder mobiles = new StringBuilder();
            String content = templateInfo.getUpContent();
            for (Map.Entry<String, List<String>> entry : entries) {
                String mobile = entry.getKey().toString();
                List<String> paramsList = entry.getValue();
                String sendMsg = getMsg(templateInfo.getUpContent(), paramsList);
                JSONObject jsonObject = new JSONObject();
                mobiles.append(mobile + ",");
                jsonObject.put("Mobile", mobile);
                jsonObject.put("Content", sendMsg);
                list.add(jsonObject);
            }
            try {
                response = bgySmsService.sendBatchSms(list, areaId, channelAccount, api, templateRecordId);
                String code = response.getCode();
                if (code.equals(ResponseCode.SUCCESS.getCode())) {
                    sysRecordInfo.setChargeNum(list.size());
                    sysRecordInfo.setMobile(mobiles.deleteCharAt(mobiles.length() - 1).toString());
                    sysRecordInfo.setContent(content);
                    sysRecordInfo.setStatus(ResponseCode.SUCCESS.getCode());
                    sysRecordInfo.setStatusDesc(response.getMsg());
                    sysRecordService.insert(sysRecordInfo);
                    return new DmHubResponse(ResponseCode.SUCCESS);
                } else {
                    sysRecordInfo.setChargeNum(list.size());
                    sysRecordInfo.setMobile(mobiles.deleteCharAt(mobiles.length() - 1).toString());
                    sysRecordInfo.setContent(content);
                    sysRecordInfo.setStatus(ResponseCode.UPSTREAM_FAIL.getCode());
                    sysRecordInfo.setStatusDesc(response.getMsg());
                    boolean a = sysRecordService.insert(sysRecordInfo);
                    return new DmHubResponse(code, response.getMsg());
                }

            } catch (Exception e) {
                log.error("调用碧桂园逻辑错误:", e);
                return new DmHubResponse("999", "插件服务系统异常");
            }
        }
        return new DmHubResponse("999", "插件服务系统异常");
    }

    private static void getKeyAndValue(JSONObject json, List<String> keyList, StringBuffer keyBuffer, Map<String, String> paramsMap) {
        Set<String> keys = json.keySet();
        for (String key : keys) {
            keyBuffer.append(".").append(key);
            Object info = json.get(key);
            StringBuffer tempInfo = new StringBuffer();

            if (info instanceof JSONObject || info instanceof Map) {
                String s = JSONObject.toJSONString(info);
                JSONObject valueJson = JSONObject.parseObject(s);
                getKeyAndValue(valueJson, keyList, keyBuffer, paramsMap);
                int key1 = keyBuffer.indexOf(key);
                String substring = keyBuffer.substring(0, key1 - 1);
                System.out.println(substring);
                keyBuffer = new StringBuffer(substring);
            } else {
                paramsMap.put(keyBuffer.toString(), String.valueOf(info));
                keyList.add(keyBuffer.toString());
                int key1 = keyBuffer.indexOf(key);
                String substring = keyBuffer.substring(0, key1 - 1);
                System.out.println(substring);
                keyBuffer = new StringBuffer(substring);
            }
        }
    }


    private String getKeyValue(Object value, StringBuilder keyBuffer) {
        String s = JSONObject.toJSONString(value);
        JSONObject valueJson = JSONObject.parseObject(s);
        Set<String> set = valueJson.keySet();
        if (set.isEmpty()) {
            return value.toString();
        }
        for (String str : set) {
            keyBuffer.append(".").append(str);
            Object info = valueJson.get(str);
            if (info instanceof JSONObject || info instanceof Map) {
                String test = getKeyValue(info, keyBuffer);
                return test;
            } else {
                return info.toString();
            }
        }
        return value.toString();
    }


}


